'use strict';

Object.defineProperty(exports, '__esModule', { value: true });

function _interopDefault (ex) { return (ex && (typeof ex === 'object') && 'default' in ex) ? ex['default'] : ex; }

var React = _interopDefault(require('react'));
var core = require('@material-ui/core');
var lodash = require('lodash');
var clsx = _interopDefault(require('clsx'));
var styles = require('@material-ui/core/styles');
var icons = require('@material-ui/icons');
var Skeleton = _interopDefault(require('@material-ui/lab/Skeleton'));
var Flatbush = _interopDefault(require('flatbush'));
var isFunction = _interopDefault(require('lodash.isfunction'));
var ResizeObserver = _interopDefault(require('resize-observer-polyfill'));

function asyncGeneratorStep(gen, resolve, reject, _next, _throw, key, arg) {
  try {
    var info = gen[key](arg);
    var value = info.value;
  } catch (error) {
    reject(error);
    return;
  }

  if (info.done) {
    resolve(value);
  } else {
    Promise.resolve(value).then(_next, _throw);
  }
}

function _asyncToGenerator(fn) {
  return function () {
    var self = this,
        args = arguments;
    return new Promise(function (resolve, reject) {
      var gen = fn.apply(self, args);

      function _next(value) {
        asyncGeneratorStep(gen, resolve, reject, _next, _throw, "next", value);
      }

      function _throw(err) {
        asyncGeneratorStep(gen, resolve, reject, _next, _throw, "throw", err);
      }

      _next(undefined);
    });
  };
}

function _extends() {
  _extends = Object.assign || function (target) {
    for (var i = 1; i < arguments.length; i++) {
      var source = arguments[i];

      for (var key in source) {
        if (Object.prototype.hasOwnProperty.call(source, key)) {
          target[key] = source[key];
        }
      }
    }

    return target;
  };

  return _extends.apply(this, arguments);
}

function _objectWithoutPropertiesLoose(source, excluded) {
  if (source == null) return {};
  var target = {};
  var sourceKeys = Object.keys(source);
  var key, i;

  for (i = 0; i < sourceKeys.length; i++) {
    key = sourceKeys[i];
    if (excluded.indexOf(key) >= 0) continue;
    target[key] = source[key];
  }

  return target;
}

function _unsupportedIterableToArray(o, minLen) {
  if (!o) return;
  if (typeof o === "string") return _arrayLikeToArray(o, minLen);
  var n = Object.prototype.toString.call(o).slice(8, -1);
  if (n === "Object" && o.constructor) n = o.constructor.name;
  if (n === "Map" || n === "Set") return Array.from(o);
  if (n === "Arguments" || /^(?:Ui|I)nt(?:8|16|32)(?:Clamped)?Array$/.test(n)) return _arrayLikeToArray(o, minLen);
}

function _arrayLikeToArray(arr, len) {
  if (len == null || len > arr.length) len = arr.length;

  for (var i = 0, arr2 = new Array(len); i < len; i++) arr2[i] = arr[i];

  return arr2;
}

function _createForOfIteratorHelperLoose(o, allowArrayLike) {
  var it = typeof Symbol !== "undefined" && o[Symbol.iterator] || o["@@iterator"];
  if (it) return (it = it.call(o)).next.bind(it);

  if (Array.isArray(o) || (it = _unsupportedIterableToArray(o)) || allowArrayLike && o && typeof o.length === "number") {
    if (it) o = it;
    var i = 0;
    return function () {
      if (i >= o.length) return {
        done: true
      };
      return {
        done: false,
        value: o[i++]
      };
    };
  }

  throw new TypeError("Invalid attempt to iterate non-iterable instance.\nIn order to be iterable, non-array objects must have a [Symbol.iterator]() method.");
}

var useStyles = /*#__PURE__*/styles.makeStyles(function (theme) {
  var _gridPages, _gridNavigation;

  return {
    grow: {
      flexGrow: 1
    },
    icon: {
      color: "white",
      padding: theme.spacing(1),
      "&.Mui-disabled": {
        color: theme.palette.grey[500]
      }
    },
    input: {
      fontSize: "14px",
      fontWeight: 400,
      padding: theme.spacing(1.25)
    },
    number: {
      color: "black",
      backgroundColor: "white",
      width: 36,
      "& input::-webkit-outer-spin-button, & input::-webkit-inner-spin-button": {
        "-webkit-appearance": "none",
        margin: 0
      }
    },
    numberInput: {
      "-moz-appearance": "textfield",
      textAlign: "center"
    },
    numberRoot: {
      borderRadius: 0
    },
    pages: {
      color: "white",
      fontSize: "14px",
      fontWeight: 400,
      margin: theme.spacing(0, 1)
    },
    separator: {
      color: "white",
      fontSize: "14px",
      fontWeight: 400,
      marginLeft: theme.spacing(1)
    },
    pageCount: {
      color: "white",
      fontSize: "14px",
      fontWeight: 400,
      marginRight: theme.spacing(1),
      marginLeft: theme.spacing(0.5)
    },
    title: {
      fontSize: "16px",
      fontWeight: 600
    },
    toolbar: {
      display: "flex",
      justifyContent: "space-between",
      flexFlow: "wrap"
    },
    gridTitle: {
      order: 1
    },
    gridPages: (_gridPages = {
      order: 2
    }, _gridPages[theme.breakpoints.down("sm")] = {
      order: 3
    }, _gridPages),
    gridNavigation: (_gridNavigation = {
      order: 3
    }, _gridNavigation[theme.breakpoints.down("sm")] = {
      order: 2
    }, _gridNavigation),
    sourceLink: {
      marginRight: theme.spacing(1)
    }
  };
});

var NavBar = function NavBar(props) {
  var documentName = props.documentName,
      SourceLink = props.SourceLink,
      navigationIndex = props.navigationIndex,
      navigationTotal = props.navigationTotal,
      onClose = props.onClose,
      onNavigationIndexChange = props.onNavigationIndexChange,
      onNextDocument = props.onNextDocument,
      onPreviousDocument = props.onPreviousDocument;
  var classes = useStyles();
  var inputRef = React.useRef(null);

  var _React$useState = React.useState(navigationIndex.toString()),
      inputValue = _React$useState[0],
      setInputValue = _React$useState[1];

  var handleNavigationItemUp = function handleNavigationItemUp() {
    onNavigationIndexChange(navigationIndex - 1);
  };

  var handleNavigationItemDown = function handleNavigationItemDown() {
    onNavigationIndexChange(navigationIndex + 1);
  };

  var handleNavigationItemBlur = React.useCallback(function () {
    setInputValue(navigationIndex.toString());
  }, [navigationIndex]);
  var confirmInput = React.useMemo(function () {
    return lodash.debounce(function () {
      if (!inputRef || !inputRef.current) return;
      var value = parseInt(inputRef.current.value);

      if (value > 0 && value <= navigationTotal) {
        onNavigationIndexChange(value);
      }
    }, 750);
  }, [navigationTotal, onNavigationIndexChange]);

  var handleNavigationItemChange = function handleNavigationItemChange(event) {
    setInputValue(event.target.value);
    confirmInput();
  };

  var handleNavigationItemKeyDown = React.useCallback(function (event) {
    if (event.key !== "Enter") return;
    confirmInput.flush();
  }, [confirmInput]);
  React.useEffect(function () {
    setInputValue(navigationIndex.toString());
  }, [navigationIndex]);
  return React.createElement(core.Toolbar, {
    className: classes.toolbar
  }, React.createElement(core.Grid, {
    className: classes.gridTitle,
    item: true,
    md: 4,
    xs: 6
  }, React.createElement(core.Typography, {
    className: classes.title,
    variant: "h3",
    color: "inherit",
    noWrap: true
  }, documentName)), React.createElement(core.Grid, {
    className: classes.gridPages,
    container: true,
    item: true,
    md: 4,
    xs: 12,
    alignItems: "center",
    wrap: "nowrap",
    justify: "center"
  }, React.createElement(core.Typography, {
    className: classes.pages,
    variant: "body1",
    display: "inline",
    noWrap: true
  }, "Pages"), React.createElement(core.TextField, {
    className: classes.number,
    inputRef: inputRef,
    type: "number",
    onChange: handleNavigationItemChange,
    onBlur: handleNavigationItemBlur,
    onKeyDown: handleNavigationItemKeyDown,
    value: inputValue,
    inputProps: {
      min: 1,
      max: navigationTotal
    },
    InputProps: {
      classes: {
        root: classes.numberRoot,
        input: clsx(classes.input, classes.numberInput)
      }
    },
    variant: "outlined"
  }), React.createElement(core.Typography, {
    className: classes.separator,
    variant: "body1",
    display: "inline",
    noWrap: true
  }, "/"), React.createElement(core.Typography, {
    id: "page-count",
    className: classes.pageCount,
    variant: "body1",
    display: "inline",
    noWrap: true
  }, navigationTotal), React.createElement(core.IconButton, {
    "aria-label": "Previous Page",
    className: classes.icon,
    disabled: navigationIndex <= 1,
    onClick: handleNavigationItemUp
  }, React.createElement(icons.ExpandLess, null)), React.createElement(core.IconButton, {
    "aria-label": "Next Page",
    className: classes.icon,
    disabled: navigationIndex >= navigationTotal,
    onClick: handleNavigationItemDown
  }, React.createElement(icons.ExpandMore, null))), React.createElement(core.Grid, {
    className: classes.gridNavigation,
    container: true,
    item: true,
    md: 4,
    xs: 6,
    justify: "flex-end",
    alignItems: "center",
    wrap: "nowrap"
  }, SourceLink && React.createElement(core.Paper, {
    className: classes.sourceLink,
    elevation: 0
  }, SourceLink), React.createElement(core.IconButton, {
    "aria-label": "Previous Document",
    className: classes.icon,
    disabled: !onPreviousDocument,
    onClick: onPreviousDocument
  }, React.createElement(icons.ChevronLeft, null)), React.createElement(core.IconButton, {
    "aria-label": "Next Document",
    className: classes.icon,
    disabled: !onNextDocument,
    onClick: onNextDocument
  }, React.createElement(icons.ChevronRight, null)), React.createElement(core.IconButton, {
    "aria-label": "Close Document",
    color: "inherit",
    onClick: onClose
  }, React.createElement(icons.Close, null))));
};

var useStyles$1 = /*#__PURE__*/styles.makeStyles({
  root: {
    position: "fixed",
    top: "90px",
    right: "15px",
    width: "200px",
    background: "white",
    border: "solid 1px #E5E9EC",
    boxShadow: "0 1px 3px rgba(14, 15, 15, 0.3)"
  },
  header: {
    background: "#E5E9EC",
    borderBottom: "solid 1px #E5E9EC",
    padding: "12px 6px"
  },
  title: {
    margin: "0"
  },
  items: {
    overflow: "auto",
    maxHeight: "500px",
    padding: 0,
    margin: 0
  },
  item: {
    cursor: "pointer",
    padding: "6px 12px",
    margin: "3px 0",
    listStyle: "none",
    whiteSpace: "nowrap",
    overflow: "hidden",
    textOverflow: "ellipsis"
  }
});

var Picker = function Picker(props) {
  var onAnnotation = props.onAnnotation,
      topics = props.topics;
  var classes = useStyles$1();
  return React.createElement(core.Box, {
    className: clsx("Picker", classes.root)
  }, React.createElement("div", {
    className: classes.header
  }, React.createElement("h4", {
    className: classes.title
  }, "Fields")), React.createElement("ol", {
    className: classes.items
  }, topics.map(function (topic) {
    return React.createElement("li", {
      key: topic,
      className: classes.item,
      onClick: function onClick() {
        onAnnotation(topic);
      }
    }, topic);
  })));
};

function createCommonjsModule(fn, module) {
	return module = { exports: {} }, fn(module, module.exports), module.exports;
}

var runtime_1 = createCommonjsModule(function (module) {
/**
 * Copyright (c) 2014-present, Facebook, Inc.
 *
 * This source code is licensed under the MIT license found in the
 * LICENSE file in the root directory of this source tree.
 */

var runtime = (function (exports) {

  var Op = Object.prototype;
  var hasOwn = Op.hasOwnProperty;
  var undefined$1; // More compressible than void 0.
  var $Symbol = typeof Symbol === "function" ? Symbol : {};
  var iteratorSymbol = $Symbol.iterator || "@@iterator";
  var asyncIteratorSymbol = $Symbol.asyncIterator || "@@asyncIterator";
  var toStringTagSymbol = $Symbol.toStringTag || "@@toStringTag";

  function define(obj, key, value) {
    Object.defineProperty(obj, key, {
      value: value,
      enumerable: true,
      configurable: true,
      writable: true
    });
    return obj[key];
  }
  try {
    // IE 8 has a broken Object.defineProperty that only works on DOM objects.
    define({}, "");
  } catch (err) {
    define = function(obj, key, value) {
      return obj[key] = value;
    };
  }

  function wrap(innerFn, outerFn, self, tryLocsList) {
    // If outerFn provided and outerFn.prototype is a Generator, then outerFn.prototype instanceof Generator.
    var protoGenerator = outerFn && outerFn.prototype instanceof Generator ? outerFn : Generator;
    var generator = Object.create(protoGenerator.prototype);
    var context = new Context(tryLocsList || []);

    // The ._invoke method unifies the implementations of the .next,
    // .throw, and .return methods.
    generator._invoke = makeInvokeMethod(innerFn, self, context);

    return generator;
  }
  exports.wrap = wrap;

  // Try/catch helper to minimize deoptimizations. Returns a completion
  // record like context.tryEntries[i].completion. This interface could
  // have been (and was previously) designed to take a closure to be
  // invoked without arguments, but in all the cases we care about we
  // already have an existing method we want to call, so there's no need
  // to create a new function object. We can even get away with assuming
  // the method takes exactly one argument, since that happens to be true
  // in every case, so we don't have to touch the arguments object. The
  // only additional allocation required is the completion record, which
  // has a stable shape and so hopefully should be cheap to allocate.
  function tryCatch(fn, obj, arg) {
    try {
      return { type: "normal", arg: fn.call(obj, arg) };
    } catch (err) {
      return { type: "throw", arg: err };
    }
  }

  var GenStateSuspendedStart = "suspendedStart";
  var GenStateSuspendedYield = "suspendedYield";
  var GenStateExecuting = "executing";
  var GenStateCompleted = "completed";

  // Returning this object from the innerFn has the same effect as
  // breaking out of the dispatch switch statement.
  var ContinueSentinel = {};

  // Dummy constructor functions that we use as the .constructor and
  // .constructor.prototype properties for functions that return Generator
  // objects. For full spec compliance, you may wish to configure your
  // minifier not to mangle the names of these two functions.
  function Generator() {}
  function GeneratorFunction() {}
  function GeneratorFunctionPrototype() {}

  // This is a polyfill for %IteratorPrototype% for environments that
  // don't natively support it.
  var IteratorPrototype = {};
  IteratorPrototype[iteratorSymbol] = function () {
    return this;
  };

  var getProto = Object.getPrototypeOf;
  var NativeIteratorPrototype = getProto && getProto(getProto(values([])));
  if (NativeIteratorPrototype &&
      NativeIteratorPrototype !== Op &&
      hasOwn.call(NativeIteratorPrototype, iteratorSymbol)) {
    // This environment has a native %IteratorPrototype%; use it instead
    // of the polyfill.
    IteratorPrototype = NativeIteratorPrototype;
  }

  var Gp = GeneratorFunctionPrototype.prototype =
    Generator.prototype = Object.create(IteratorPrototype);
  GeneratorFunction.prototype = Gp.constructor = GeneratorFunctionPrototype;
  GeneratorFunctionPrototype.constructor = GeneratorFunction;
  GeneratorFunction.displayName = define(
    GeneratorFunctionPrototype,
    toStringTagSymbol,
    "GeneratorFunction"
  );

  // Helper for defining the .next, .throw, and .return methods of the
  // Iterator interface in terms of a single ._invoke method.
  function defineIteratorMethods(prototype) {
    ["next", "throw", "return"].forEach(function(method) {
      define(prototype, method, function(arg) {
        return this._invoke(method, arg);
      });
    });
  }

  exports.isGeneratorFunction = function(genFun) {
    var ctor = typeof genFun === "function" && genFun.constructor;
    return ctor
      ? ctor === GeneratorFunction ||
        // For the native GeneratorFunction constructor, the best we can
        // do is to check its .name property.
        (ctor.displayName || ctor.name) === "GeneratorFunction"
      : false;
  };

  exports.mark = function(genFun) {
    if (Object.setPrototypeOf) {
      Object.setPrototypeOf(genFun, GeneratorFunctionPrototype);
    } else {
      genFun.__proto__ = GeneratorFunctionPrototype;
      define(genFun, toStringTagSymbol, "GeneratorFunction");
    }
    genFun.prototype = Object.create(Gp);
    return genFun;
  };

  // Within the body of any async function, `await x` is transformed to
  // `yield regeneratorRuntime.awrap(x)`, so that the runtime can test
  // `hasOwn.call(value, "__await")` to determine if the yielded value is
  // meant to be awaited.
  exports.awrap = function(arg) {
    return { __await: arg };
  };

  function AsyncIterator(generator, PromiseImpl) {
    function invoke(method, arg, resolve, reject) {
      var record = tryCatch(generator[method], generator, arg);
      if (record.type === "throw") {
        reject(record.arg);
      } else {
        var result = record.arg;
        var value = result.value;
        if (value &&
            typeof value === "object" &&
            hasOwn.call(value, "__await")) {
          return PromiseImpl.resolve(value.__await).then(function(value) {
            invoke("next", value, resolve, reject);
          }, function(err) {
            invoke("throw", err, resolve, reject);
          });
        }

        return PromiseImpl.resolve(value).then(function(unwrapped) {
          // When a yielded Promise is resolved, its final value becomes
          // the .value of the Promise<{value,done}> result for the
          // current iteration.
          result.value = unwrapped;
          resolve(result);
        }, function(error) {
          // If a rejected Promise was yielded, throw the rejection back
          // into the async generator function so it can be handled there.
          return invoke("throw", error, resolve, reject);
        });
      }
    }

    var previousPromise;

    function enqueue(method, arg) {
      function callInvokeWithMethodAndArg() {
        return new PromiseImpl(function(resolve, reject) {
          invoke(method, arg, resolve, reject);
        });
      }

      return previousPromise =
        // If enqueue has been called before, then we want to wait until
        // all previous Promises have been resolved before calling invoke,
        // so that results are always delivered in the correct order. If
        // enqueue has not been called before, then it is important to
        // call invoke immediately, without waiting on a callback to fire,
        // so that the async generator function has the opportunity to do
        // any necessary setup in a predictable way. This predictability
        // is why the Promise constructor synchronously invokes its
        // executor callback, and why async functions synchronously
        // execute code before the first await. Since we implement simple
        // async functions in terms of async generators, it is especially
        // important to get this right, even though it requires care.
        previousPromise ? previousPromise.then(
          callInvokeWithMethodAndArg,
          // Avoid propagating failures to Promises returned by later
          // invocations of the iterator.
          callInvokeWithMethodAndArg
        ) : callInvokeWithMethodAndArg();
    }

    // Define the unified helper method that is used to implement .next,
    // .throw, and .return (see defineIteratorMethods).
    this._invoke = enqueue;
  }

  defineIteratorMethods(AsyncIterator.prototype);
  AsyncIterator.prototype[asyncIteratorSymbol] = function () {
    return this;
  };
  exports.AsyncIterator = AsyncIterator;

  // Note that simple async functions are implemented on top of
  // AsyncIterator objects; they just return a Promise for the value of
  // the final result produced by the iterator.
  exports.async = function(innerFn, outerFn, self, tryLocsList, PromiseImpl) {
    if (PromiseImpl === void 0) PromiseImpl = Promise;

    var iter = new AsyncIterator(
      wrap(innerFn, outerFn, self, tryLocsList),
      PromiseImpl
    );

    return exports.isGeneratorFunction(outerFn)
      ? iter // If outerFn is a generator, return the full iterator.
      : iter.next().then(function(result) {
          return result.done ? result.value : iter.next();
        });
  };

  function makeInvokeMethod(innerFn, self, context) {
    var state = GenStateSuspendedStart;

    return function invoke(method, arg) {
      if (state === GenStateExecuting) {
        throw new Error("Generator is already running");
      }

      if (state === GenStateCompleted) {
        if (method === "throw") {
          throw arg;
        }

        // Be forgiving, per 25.3.3.3.3 of the spec:
        // https://people.mozilla.org/~jorendorff/es6-draft.html#sec-generatorresume
        return doneResult();
      }

      context.method = method;
      context.arg = arg;

      while (true) {
        var delegate = context.delegate;
        if (delegate) {
          var delegateResult = maybeInvokeDelegate(delegate, context);
          if (delegateResult) {
            if (delegateResult === ContinueSentinel) continue;
            return delegateResult;
          }
        }

        if (context.method === "next") {
          // Setting context._sent for legacy support of Babel's
          // function.sent implementation.
          context.sent = context._sent = context.arg;

        } else if (context.method === "throw") {
          if (state === GenStateSuspendedStart) {
            state = GenStateCompleted;
            throw context.arg;
          }

          context.dispatchException(context.arg);

        } else if (context.method === "return") {
          context.abrupt("return", context.arg);
        }

        state = GenStateExecuting;

        var record = tryCatch(innerFn, self, context);
        if (record.type === "normal") {
          // If an exception is thrown from innerFn, we leave state ===
          // GenStateExecuting and loop back for another invocation.
          state = context.done
            ? GenStateCompleted
            : GenStateSuspendedYield;

          if (record.arg === ContinueSentinel) {
            continue;
          }

          return {
            value: record.arg,
            done: context.done
          };

        } else if (record.type === "throw") {
          state = GenStateCompleted;
          // Dispatch the exception by looping back around to the
          // context.dispatchException(context.arg) call above.
          context.method = "throw";
          context.arg = record.arg;
        }
      }
    };
  }

  // Call delegate.iterator[context.method](context.arg) and handle the
  // result, either by returning a { value, done } result from the
  // delegate iterator, or by modifying context.method and context.arg,
  // setting context.delegate to null, and returning the ContinueSentinel.
  function maybeInvokeDelegate(delegate, context) {
    var method = delegate.iterator[context.method];
    if (method === undefined$1) {
      // A .throw or .return when the delegate iterator has no .throw
      // method always terminates the yield* loop.
      context.delegate = null;

      if (context.method === "throw") {
        // Note: ["return"] must be used for ES3 parsing compatibility.
        if (delegate.iterator["return"]) {
          // If the delegate iterator has a return method, give it a
          // chance to clean up.
          context.method = "return";
          context.arg = undefined$1;
          maybeInvokeDelegate(delegate, context);

          if (context.method === "throw") {
            // If maybeInvokeDelegate(context) changed context.method from
            // "return" to "throw", let that override the TypeError below.
            return ContinueSentinel;
          }
        }

        context.method = "throw";
        context.arg = new TypeError(
          "The iterator does not provide a 'throw' method");
      }

      return ContinueSentinel;
    }

    var record = tryCatch(method, delegate.iterator, context.arg);

    if (record.type === "throw") {
      context.method = "throw";
      context.arg = record.arg;
      context.delegate = null;
      return ContinueSentinel;
    }

    var info = record.arg;

    if (! info) {
      context.method = "throw";
      context.arg = new TypeError("iterator result is not an object");
      context.delegate = null;
      return ContinueSentinel;
    }

    if (info.done) {
      // Assign the result of the finished delegate to the temporary
      // variable specified by delegate.resultName (see delegateYield).
      context[delegate.resultName] = info.value;

      // Resume execution at the desired location (see delegateYield).
      context.next = delegate.nextLoc;

      // If context.method was "throw" but the delegate handled the
      // exception, let the outer generator proceed normally. If
      // context.method was "next", forget context.arg since it has been
      // "consumed" by the delegate iterator. If context.method was
      // "return", allow the original .return call to continue in the
      // outer generator.
      if (context.method !== "return") {
        context.method = "next";
        context.arg = undefined$1;
      }

    } else {
      // Re-yield the result returned by the delegate method.
      return info;
    }

    // The delegate iterator is finished, so forget it and continue with
    // the outer generator.
    context.delegate = null;
    return ContinueSentinel;
  }

  // Define Generator.prototype.{next,throw,return} in terms of the
  // unified ._invoke helper method.
  defineIteratorMethods(Gp);

  define(Gp, toStringTagSymbol, "Generator");

  // A Generator should always return itself as the iterator object when the
  // @@iterator function is called on it. Some browsers' implementations of the
  // iterator prototype chain incorrectly implement this, causing the Generator
  // object to not be returned from this call. This ensures that doesn't happen.
  // See https://github.com/facebook/regenerator/issues/274 for more details.
  Gp[iteratorSymbol] = function() {
    return this;
  };

  Gp.toString = function() {
    return "[object Generator]";
  };

  function pushTryEntry(locs) {
    var entry = { tryLoc: locs[0] };

    if (1 in locs) {
      entry.catchLoc = locs[1];
    }

    if (2 in locs) {
      entry.finallyLoc = locs[2];
      entry.afterLoc = locs[3];
    }

    this.tryEntries.push(entry);
  }

  function resetTryEntry(entry) {
    var record = entry.completion || {};
    record.type = "normal";
    delete record.arg;
    entry.completion = record;
  }

  function Context(tryLocsList) {
    // The root entry object (effectively a try statement without a catch
    // or a finally block) gives us a place to store values thrown from
    // locations where there is no enclosing try statement.
    this.tryEntries = [{ tryLoc: "root" }];
    tryLocsList.forEach(pushTryEntry, this);
    this.reset(true);
  }

  exports.keys = function(object) {
    var keys = [];
    for (var key in object) {
      keys.push(key);
    }
    keys.reverse();

    // Rather than returning an object with a next method, we keep
    // things simple and return the next function itself.
    return function next() {
      while (keys.length) {
        var key = keys.pop();
        if (key in object) {
          next.value = key;
          next.done = false;
          return next;
        }
      }

      // To avoid creating an additional object, we just hang the .value
      // and .done properties off the next function object itself. This
      // also ensures that the minifier will not anonymize the function.
      next.done = true;
      return next;
    };
  };

  function values(iterable) {
    if (iterable) {
      var iteratorMethod = iterable[iteratorSymbol];
      if (iteratorMethod) {
        return iteratorMethod.call(iterable);
      }

      if (typeof iterable.next === "function") {
        return iterable;
      }

      if (!isNaN(iterable.length)) {
        var i = -1, next = function next() {
          while (++i < iterable.length) {
            if (hasOwn.call(iterable, i)) {
              next.value = iterable[i];
              next.done = false;
              return next;
            }
          }

          next.value = undefined$1;
          next.done = true;

          return next;
        };

        return next.next = next;
      }
    }

    // Return an iterator with no values.
    return { next: doneResult };
  }
  exports.values = values;

  function doneResult() {
    return { value: undefined$1, done: true };
  }

  Context.prototype = {
    constructor: Context,

    reset: function(skipTempReset) {
      this.prev = 0;
      this.next = 0;
      // Resetting context._sent for legacy support of Babel's
      // function.sent implementation.
      this.sent = this._sent = undefined$1;
      this.done = false;
      this.delegate = null;

      this.method = "next";
      this.arg = undefined$1;

      this.tryEntries.forEach(resetTryEntry);

      if (!skipTempReset) {
        for (var name in this) {
          // Not sure about the optimal order of these conditions:
          if (name.charAt(0) === "t" &&
              hasOwn.call(this, name) &&
              !isNaN(+name.slice(1))) {
            this[name] = undefined$1;
          }
        }
      }
    },

    stop: function() {
      this.done = true;

      var rootEntry = this.tryEntries[0];
      var rootRecord = rootEntry.completion;
      if (rootRecord.type === "throw") {
        throw rootRecord.arg;
      }

      return this.rval;
    },

    dispatchException: function(exception) {
      if (this.done) {
        throw exception;
      }

      var context = this;
      function handle(loc, caught) {
        record.type = "throw";
        record.arg = exception;
        context.next = loc;

        if (caught) {
          // If the dispatched exception was caught by a catch block,
          // then let that catch block handle the exception normally.
          context.method = "next";
          context.arg = undefined$1;
        }

        return !! caught;
      }

      for (var i = this.tryEntries.length - 1; i >= 0; --i) {
        var entry = this.tryEntries[i];
        var record = entry.completion;

        if (entry.tryLoc === "root") {
          // Exception thrown outside of any try block that could handle
          // it, so set the completion value of the entire function to
          // throw the exception.
          return handle("end");
        }

        if (entry.tryLoc <= this.prev) {
          var hasCatch = hasOwn.call(entry, "catchLoc");
          var hasFinally = hasOwn.call(entry, "finallyLoc");

          if (hasCatch && hasFinally) {
            if (this.prev < entry.catchLoc) {
              return handle(entry.catchLoc, true);
            } else if (this.prev < entry.finallyLoc) {
              return handle(entry.finallyLoc);
            }

          } else if (hasCatch) {
            if (this.prev < entry.catchLoc) {
              return handle(entry.catchLoc, true);
            }

          } else if (hasFinally) {
            if (this.prev < entry.finallyLoc) {
              return handle(entry.finallyLoc);
            }

          } else {
            throw new Error("try statement without catch or finally");
          }
        }
      }
    },

    abrupt: function(type, arg) {
      for (var i = this.tryEntries.length - 1; i >= 0; --i) {
        var entry = this.tryEntries[i];
        if (entry.tryLoc <= this.prev &&
            hasOwn.call(entry, "finallyLoc") &&
            this.prev < entry.finallyLoc) {
          var finallyEntry = entry;
          break;
        }
      }

      if (finallyEntry &&
          (type === "break" ||
           type === "continue") &&
          finallyEntry.tryLoc <= arg &&
          arg <= finallyEntry.finallyLoc) {
        // Ignore the finally entry if control is not jumping to a
        // location outside the try/catch block.
        finallyEntry = null;
      }

      var record = finallyEntry ? finallyEntry.completion : {};
      record.type = type;
      record.arg = arg;

      if (finallyEntry) {
        this.method = "next";
        this.next = finallyEntry.finallyLoc;
        return ContinueSentinel;
      }

      return this.complete(record);
    },

    complete: function(record, afterLoc) {
      if (record.type === "throw") {
        throw record.arg;
      }

      if (record.type === "break" ||
          record.type === "continue") {
        this.next = record.arg;
      } else if (record.type === "return") {
        this.rval = this.arg = record.arg;
        this.method = "return";
        this.next = "end";
      } else if (record.type === "normal" && afterLoc) {
        this.next = afterLoc;
      }

      return ContinueSentinel;
    },

    finish: function(finallyLoc) {
      for (var i = this.tryEntries.length - 1; i >= 0; --i) {
        var entry = this.tryEntries[i];
        if (entry.finallyLoc === finallyLoc) {
          this.complete(entry.completion, entry.afterLoc);
          resetTryEntry(entry);
          return ContinueSentinel;
        }
      }
    },

    "catch": function(tryLoc) {
      for (var i = this.tryEntries.length - 1; i >= 0; --i) {
        var entry = this.tryEntries[i];
        if (entry.tryLoc === tryLoc) {
          var record = entry.completion;
          if (record.type === "throw") {
            var thrown = record.arg;
            resetTryEntry(entry);
          }
          return thrown;
        }
      }

      // The context.catch method must only be called with a location
      // argument that corresponds to a known catch block.
      throw new Error("illegal catch attempt");
    },

    delegateYield: function(iterable, resultName, nextLoc) {
      this.delegate = {
        iterator: values(iterable),
        resultName: resultName,
        nextLoc: nextLoc
      };

      if (this.method === "next") {
        // Deliberately forget the last sent value so that we don't
        // accidentally pass it on to the delegate.
        this.arg = undefined$1;
      }

      return ContinueSentinel;
    }
  };

  // Regardless of whether this script is executing as a CommonJS module
  // or not, return the runtime object so that we can declare the variable
  // regeneratorRuntime in the outer scope, which allows this module to be
  // injected easily by `bin/regenerator --include-runtime script.js`.
  return exports;

}(
  // If this script is executing as a CommonJS module, use module.exports
  // as the regeneratorRuntime namespace. Otherwise create a new empty
  // object. Either way, the resulting object will be used to initialize
  // the regeneratorRuntime variable at the top of this file.
   module.exports 
));

try {
  regeneratorRuntime = runtime;
} catch (accidentalStrictMode) {
  // This module should not be running in strict mode, so the above
  // assignment should always work unless something is misconfigured. Just
  // in case runtime.js accidentally runs in strict mode, we can escape
  // strict mode using a global Function call. This could conceivably fail
  // if a Content Security Policy forbids using Function, but in that case
  // the proper solution is to fix the accidental strict mode problem. If
  // you've misconfigured your bundler to force strict mode and applied a
  // CSP to forbid Function, and you're not willing to fix either of those
  // problems, please detail your unique predicament in a GitHub issue.
  Function("r", "regeneratorRuntime = r")(runtime);
}
});

var ANNOTATION_COLORS = ["#a6cee3", "#1f78b4", "#b2df8a", "#33a02c", "#fb9a99", "#e31a1c", "#fdbf6f", "#ff7f00", "#cab2d6", "#6a3d9a", "#ff9", "#b15928"];

function hashString(str) {
  return str.split("").reduce(function (a, b) {
    a = (a << 5) - a + b.charCodeAt(0);
    return a & a;
  }, 0);
}

function topicToColor(topic) {
  return ANNOTATION_COLORS[Math.abs(hashString(topic) % ANNOTATION_COLORS.length)];
}

var RECTANGLE_PADDING = 6; // px

var useStyles$2 = /*#__PURE__*/styles.makeStyles({
  root: {
    position: "relative",
    flex: "1 1 auto",
    cursor: "text"
  },
  image: {
    height: "100%",
    width: "100%"
  },
  svg: {
    position: "absolute",
    top: "0px",
    left: "0px",
    height: "100%",
    width: "100%"
  },
  selection: {
    mixBlendMode: "multiply",
    fillOpacity: 0.6
  },
  searchResult: {
    fill: "#E9CB77",
    fillOpacity: 0.5,
    mixBlendMode: "multiply"
  },
  rect: {
    cursor: "pointer",
    mixBlendMode: "multiply",
    fillOpacity: 0.3
  },
  rectFocused: {
    fillOpacity: 0.6
  }
}); // Returns closest
// var arr = [{characterStart: 1, characterEnd: 5},
//   {characterStart: 5, characterEnd: 10},
//   {characterStart: 10, characterEnd: 15},
//   {characterStart: 15, characterEnd: 20}];
// [-1, 0, 1, 2, 3, 4, 5, 6, 7, 8, 9, 10, 11, 12, 13, 14, 15, 16, 17, 18, 19, 20, 21, 100].forEach((n: number) =>
// console.log(characterStartToTokenIndex(arr, n))
// );

var characterStartToTokenIndex = function characterStartToTokenIndex(characterStart, tokens) {
  var mid,
      left = 0,
      right = tokens.length - 1;

  while (left <= right) {
    mid = Math.floor((left + right) / 2);

    if (characterStart < tokens[mid].characterStart) {
      right = mid - 1;
    } else if (characterStart >= tokens[mid].characterEnd) {
      // exclusive end
      left = mid + 1;
    } else {
      return mid;
    }
  }

  return left < tokens.length - 1 ? left : tokens.length - 1;
}; // Returns closest
// var arr = [{characterStart: 1, characterEnd: 5},
//   {characterStart: 5, characterEnd: 10},
//   {characterStart: 10, characterEnd: 15},
//   {characterStart: 15, characterEnd: 20}];
// [-1, 0, 1, 2, 3, 4, 5, 6, 7, 8, 9, 10, 11, 12, 13, 14, 15, 16, 17, 18, 19, 20, 21, 100].forEach((n: number) =>
// console.log(characterEndToTokenIndex(arr, n))
// );


var characterEndToTokenIndex = function characterEndToTokenIndex(characterEnd, tokens) {
  var mid,
      left = 0,
      right = tokens.length - 1;

  while (left <= right) {
    mid = Math.floor((left + right) / 2);

    if (characterEnd <= tokens[mid].characterStart) {
      // exclusive end
      right = mid - 1;
    } else if (characterEnd > tokens[mid].characterEnd) {
      // exclusive end
      left = mid + 1;
    } else {
      return mid;
    }
  }

  return right < 0 ? 0 : right;
};

var annotationToTokens = function annotationToTokens(annotation, tokens) {
  var startIndex = annotation.characterStart === Number.NEGATIVE_INFINITY ? 0 : characterStartToTokenIndex(annotation.characterStart, tokens);
  var endIndex = annotation.characterEnd === Number.POSITIVE_INFINITY ? tokens.length - 1 : characterEndToTokenIndex(annotation.characterEnd, tokens);
  return tokens.slice(startIndex, endIndex + 1);
};

var tokensToRectangles = function tokensToRectangles(tokens) {
  var lines = tokens.reduce(function (linesMap, token) {
    var _linesMap$get;

    var lineTokens = (_linesMap$get = linesMap.get(token.line)) != null ? _linesMap$get : [];
    lineTokens.push(token);
    linesMap.set(token.line, lineTokens);
    return linesMap;
  }, new Map());
  var rectangles = [];

  for (var _i = 0, _Array$from = Array.from(lines.values()); _i < _Array$from.length; _i++) {
    var lineTokens = _Array$from[_i];
    var rectangle = lineTokens.map(function (token) {
      return _extends({}, token.boundingBox);
    }).reduce(function (boundingBox, tokenBoundingBox) {
      boundingBox.top = Math.min(boundingBox.top, tokenBoundingBox.top);
      boundingBox.bottom = Math.max(boundingBox.bottom, tokenBoundingBox.bottom);
      boundingBox.left = Math.min(boundingBox.left, tokenBoundingBox.left);
      boundingBox.right = Math.max(boundingBox.right, tokenBoundingBox.right);
      return boundingBox;
    });
    rectangles.push(rectangle);
  }

  return rectangles;
};

var annotationToRectangles = function annotationToRectangles(annotation, tokens) {
  var annotationTokens = annotationToTokens(annotation, tokens);
  return tokensToRectangles(annotationTokens);
}; // var tokens = [{characterStart: 1, characterEnd: 5, line: 0, boundingBox: {top: 0, bottom: 10, left: 0, right: 10}},
//   {characterStart: 5, characterEnd: 10, line: 0, boundingBox: {top: 0, bottom: 15, left: 15, right: 25}},
//   {characterStart: 10, characterEnd: 15, line: 1, boundingBox: {top: 15, bottom: 25, left: 0, right: 10}},
//   {characterStart: 15, characterEnd: 20, line: 1, boundingBox: {top: 15, bottom: 25, left: 15, right: 25}}];
// var annotations = [{characterStart: 1, characterEnd: 20}];
// console.log(annotationsToRectangles(annotations, tokens));


var annotationsToRectangles = function annotationsToRectangles(annotations, tokens) {
  if (!tokens) return [];
  return annotations.reduce(function (rectangles, annotation) {
    var annotationRectangles = annotationToRectangles(annotation, tokens).map(function (rectangle) {
      return _extends({}, rectangle, {
        annotationTopic: annotation.topic,
        annotationIndex: annotation._index
      });
    });
    return rectangles.concat(annotationRectangles);
  }, []);
};

var searchResultsToRectangles = function searchResultsToRectangles(searchResults, tokens) {
  if (!tokens) return [];
  return searchResults.map(function (searchResult) {
    var startIndex = searchResult.characterStart === Number.NEGATIVE_INFINITY ? 0 : characterStartToTokenIndex(searchResult.characterStart, tokens);
    var endIndex = searchResult.characterEnd === Number.POSITIVE_INFINITY ? tokens.length - 1 : characterEndToTokenIndex(searchResult.characterEnd, tokens);
    var searchResultTokens = tokens.slice(startIndex, endIndex + 1);
    return tokensToRectangles(searchResultTokens);
  }).flat(1);
};

var selectionToRectangles = function selectionToRectangles(selection, tokens) {
  if (!selection || !tokens) return [];
  var startIndex = selection.indexStart === Number.NEGATIVE_INFINITY ? 0 : selection.indexStart;
  var endIndex = selection.indexEnd === Number.POSITIVE_INFINITY ? tokens.length - 1 : selection.indexEnd;
  var selectedTokens = tokens.slice(startIndex, endIndex + 1);
  return tokensToRectangles(selectedTokens);
};

var SelectionInPage;

(function (SelectionInPage) {
  SelectionInPage[SelectionInPage["Start"] = 0] = "Start";
  SelectionInPage[SelectionInPage["End"] = 1] = "End";
  SelectionInPage[SelectionInPage["Both"] = 2] = "Both";
  SelectionInPage[SelectionInPage["None"] = 3] = "None";
})(SelectionInPage || (SelectionInPage = {}));

var selectionInPage = function selectionInPage(mouseSelection, pageImageWrapperEl) {
  var containerEl = pageImageWrapperEl.closest(".Pages");
  if (!containerEl || !mouseSelection) return SelectionInPage.None;
  var containerBCR = containerEl.getBoundingClientRect();
  var imageBCR = pageImageWrapperEl.getBoundingClientRect();
  var top = containerEl.scrollTop + (imageBCR.top - containerBCR.top);
  var bottom = top + imageBCR.height;
  var _mouseSelection$ = mouseSelection[0],
      y1 = _mouseSelection$[1],
      _mouseSelection$2 = mouseSelection[1],
      y2 = _mouseSelection$2[1];
  if (top <= y1 && y1 <= y2 && y2 <= bottom) return SelectionInPage.Both;
  if (top <= y1 && y1 <= bottom) return SelectionInPage.Start;
  if (top <= y2 && y2 <= bottom) return SelectionInPage.End;
  return SelectionInPage.None;
};

var scaleMousePosition = function scaleMousePosition(mousePosition, originalPageHeight, originalPageWidth, pageImageWrapperEl) {
  var containerEl = pageImageWrapperEl.closest(".Pages");
  if (!mousePosition || !containerEl) return [0, 0];
  var containerBCR = containerEl.getBoundingClientRect();
  var pageBCR = pageImageWrapperEl.getBoundingClientRect();
  var x = mousePosition[0];
  var y = mousePosition[1];
  var scaledX = (x - (pageBCR.left - containerBCR.left + containerEl.scrollLeft)) * (originalPageWidth / pageBCR.width);
  var scaledY = (y - (pageBCR.top - containerBCR.top + containerEl.scrollTop)) * (originalPageHeight / pageBCR.height);
  return [scaledX, scaledY];
};

var Viewbox = function Viewbox(props) {
  var annotations = props.annotations,
      searchResults = props.searchResults,
      focusedAnnotationIndex = props.focusedAnnotationIndex,
      focusingAnnotation = props.focusingAnnotation,
      imageURL = props.imageURL,
      onAnnotationFocus = props.onAnnotationFocus,
      onSelectionStart = props.onSelectionStart,
      onSelectionEnd = props.onSelectionEnd,
      originalPageHeight = props.originalPageHeight,
      originalPageWidth = props.originalPageWidth,
      pageNumber = props.pageNumber,
      mouseSelection = props.mouseSelection,
      selection = props.selection,
      tokensURL = props.tokensURL;
  var classes = useStyles$2();
  var pageImageWrapperRef = React.useRef(null);

  var _React$useState = React.useState(null),
      imageSrc = _React$useState[0],
      setImageSrc = _React$useState[1];

  var _React$useState2 = React.useState(null),
      tokens = _React$useState2[0],
      setTokens = _React$useState2[1];

  var _React$useState3 = React.useState(null),
      rtree = _React$useState3[0],
      setRtree = _React$useState3[1];

  var annotationRectangles = React.useMemo(function () {
    return annotationsToRectangles(annotations, tokens);
  }, [annotations, tokens]);
  var searchResultRectangles = React.useMemo(function () {
    return searchResultsToRectangles(searchResults, tokens);
  }, [searchResults, tokens]);
  var selectionRectangles = React.useMemo(function () {
    return selectionToRectangles(selection, tokens);
  }, [selection, tokens]); // Create a new AbortController for each new page or else
  // it will be aborted and fetch won't work

  var abortController = React.useMemo(function () {
    return new AbortController(); // eslint-disable-next-line react-hooks/exhaustive-deps
  }, [tokensURL]);
  React.useEffect(function () {
    function fetchTokens() {
      return _fetchTokens.apply(this, arguments);
    }

    function _fetchTokens() {
      _fetchTokens = _asyncToGenerator( /*#__PURE__*/runtime_1.mark(function _callee() {
        return runtime_1.wrap(function _callee$(_context) {
          while (1) {
            switch (_context.prev = _context.next) {
              case 0:
                _context.prev = 0;

                if (!isFunction(tokensURL)) {
                  _context.next = 5;
                  break;
                }

                _context.t0 = tokensURL();
                _context.next = 8;
                break;

              case 5:
                _context.next = 7;
                return fetch(tokensURL, {
                  cache: "force-cache",
                  signal: abortController.signal
                });

              case 7:
                _context.t0 = _context.sent.json();

              case 8:
                _context.t0.then(function (res) {
                  return setTokens(res);
                })["catch"](function (err) {
                  return console.error("Error fetching the tokens", err);
                });

                _context.next = 14;
                break;

              case 11:
                _context.prev = 11;
                _context.t1 = _context["catch"](0);

                if (_context.t1.name !== "AbortError") {
                  console.error("Error fetching the tokens", _context.t1);
                }

              case 14:
              case "end":
                return _context.stop();
            }
          }
        }, _callee, null, [[0, 11]]);
      }));
      return _fetchTokens.apply(this, arguments);
    }

    fetchTokens();
  }, [abortController, tokensURL]);
  React.useEffect(function () {
    if (!tokens || tokens.length === 0) return;
    var index = new Flatbush(tokens.length);

    for (var _iterator = _createForOfIteratorHelperLoose(tokens), _step; !(_step = _iterator()).done;) {
      var token = _step.value;
      var boundingBox = token.boundingBox;
      index.add(boundingBox.left, boundingBox.top, boundingBox.right, boundingBox.bottom);
    }

    index.finish();
    setRtree(index);
  }, [tokens]);
  React.useEffect(function () {
    if (isFunction(imageURL)) {
      imageURL().then(function (res) {
        return setImageSrc(res);
      })["catch"](function (err) {
        return console.error("Error getting the image url", err);
      });
    } else {
      setImageSrc(imageURL);
    }
  }, [imageURL]);
  React.useEffect(function () {
    if (!rtree || !pageImageWrapperRef || !pageImageWrapperRef.current || !mouseSelection || !tokens) return;

    switch (selectionInPage(mouseSelection, pageImageWrapperRef.current)) {
      case SelectionInPage.Start:
        {
          var position = scaleMousePosition(mouseSelection[0], originalPageHeight, originalPageWidth, pageImageWrapperRef.current);
          if (!position) break;
          var x1 = position[0],
              y1 = position[1];
          var intersectingIndices = rtree.search(x1, y1, Number.POSITIVE_INFINITY, Number.POSITIVE_INFINITY);
          var firstIndex = Math.min.apply(Math, intersectingIndices);

          if (Number.isInteger(firstIndex)) {
            onSelectionStart({
              index: firstIndex,
              page: pageNumber,
              token: tokens[firstIndex]
            });
          }

          break;
        }

      case SelectionInPage.End:
        {
          var _position = scaleMousePosition(mouseSelection[1], originalPageHeight, originalPageWidth, pageImageWrapperRef.current);

          if (!_position) break;
          var x2 = _position[0],
              y2 = _position[1];

          var _intersectingIndices = rtree.search(Number.NEGATIVE_INFINITY, Number.NEGATIVE_INFINITY, x2, y2);

          var lastIndex = Math.max.apply(Math, _intersectingIndices); // If you started selecting on previous pages and it had no tokens,
          // we'll take the first of this page as the start

          if (!selection && Number.isInteger(lastIndex)) {
            onSelectionStart({
              index: 0,
              page: pageNumber,
              token: tokens[0]
            });
          }

          onSelectionEnd(Number.isInteger(lastIndex) ? {
            index: lastIndex,
            page: pageNumber,
            token: tokens[lastIndex]
          } : null);
          break;
        }

      case SelectionInPage.Both:
        {
          var position1 = scaleMousePosition(mouseSelection[0], originalPageHeight, originalPageWidth, pageImageWrapperRef.current);
          var position2 = scaleMousePosition(mouseSelection[1], originalPageHeight, originalPageWidth, pageImageWrapperRef.current);
          if (!position1 || !position2) break;
          var _x = position1[0],
              _y = position1[1];
          var _x2 = position2[0],
              _y2 = position2[1];

          var _intersectingIndices2 = rtree.search(_x, _y, _x2, _y2);

          var _firstIndex = Math.min.apply(Math, _intersectingIndices2);

          var _lastIndex = Math.max.apply(Math, _intersectingIndices2);

          onSelectionStart(Number.isInteger(_firstIndex) ? {
            index: _firstIndex,
            page: pageNumber,
            token: tokens[_firstIndex]
          } : null);
          onSelectionEnd(Number.isInteger(_lastIndex) ? {
            index: _lastIndex,
            page: pageNumber,
            token: tokens[_lastIndex]
          } : null);
          break;
        }
    }
  }, [mouseSelection, onSelectionStart, onSelectionEnd, originalPageHeight, originalPageWidth, pageNumber, pageImageWrapperRef, rtree, selection, tokens]);
  return React.createElement(core.Paper, {
    className: classes.root,
    ref: pageImageWrapperRef
  }, imageSrc && React.createElement("img", {
    className: classes.image,
    src: imageSrc,
    alt: "Page " + pageNumber
  }), tokens && React.createElement("svg", {
    className: classes.svg,
    viewBox: "0 0 " + originalPageWidth + " " + originalPageHeight
  }, annotationRectangles.map(function (rectangle, index) {
    return React.createElement("rect", {
      key: "annotation-" + index,
      className: clsx(classes.rect, focusingAnnotation && rectangle.annotationIndex === focusedAnnotationIndex && classes.rectFocused),
      x: rectangle.left - RECTANGLE_PADDING,
      y: rectangle.top - RECTANGLE_PADDING,
      onClick: function onClick(event) {
        event.preventDefault();
        event.stopPropagation();
        onAnnotationFocus(rectangle.annotationIndex);
      },
      width: rectangle.right - rectangle.left + 2 * RECTANGLE_PADDING,
      height: rectangle.bottom - rectangle.top + 2 * RECTANGLE_PADDING,
      style: {
        fill: topicToColor(rectangle.annotationTopic)
      }
    });
  }), searchResultRectangles.map(function (rectangle, index) {
    return React.createElement("rect", {
      key: "search-result-" + index,
      className: classes.searchResult,
      x: rectangle.left - RECTANGLE_PADDING,
      y: rectangle.top - RECTANGLE_PADDING,
      width: rectangle.right - rectangle.left + 2 * RECTANGLE_PADDING,
      height: rectangle.bottom - rectangle.top + 2 * RECTANGLE_PADDING
    });
  }), selectionRectangles.map(function (rectangle, index) {
    return React.createElement("rect", {
      key: "selection-" + index,
      className: classes.selection,
      x: rectangle.left - RECTANGLE_PADDING,
      y: rectangle.top - RECTANGLE_PADDING,
      width: rectangle.right - rectangle.left + 2 * RECTANGLE_PADDING,
      height: rectangle.bottom - rectangle.top + 2 * RECTANGLE_PADDING
    });
  })));
};

var EmptyViewbox = function EmptyViewbox() {
  var classes = useStyles$2();
  return React.createElement(core.Paper, {
    className: classes.root
  });
};

var useStyles$3 = /*#__PURE__*/styles.makeStyles(function (theme) {
  return {
    skeletonViewbox: {
      padding: theme.spacing(4),
      position: "relative",
      flex: "1 1 auto",
      cursor: "text"
    }
  };
});

var Paragraph = function Paragraph(_ref) {
  var lines = _ref.lines,
      trail = _ref.trail;
  return React.createElement(React.Fragment, null, [].concat(Array(lines)).map(function (_, idx) {
    return React.createElement(Skeleton, {
      key: idx,
      height: "2%",
      width: "100%"
    });
  }), React.createElement(Skeleton, {
    height: "2%",
    width: trail
  }));
};

var ParagraphDivider = function ParagraphDivider() {
  return React.createElement(core.Box, {
    height: "5%"
  });
};

var PageSkeleton = function PageSkeleton() {
  var classes = useStyles$3();
  return React.createElement(core.Paper, {
    className: classes.skeletonViewbox
  }, React.createElement(Skeleton, {
    height: "4%",
    width: "20%"
  }), React.createElement(ParagraphDivider, null), React.createElement(Paragraph, {
    lines: 10,
    trail: "70%"
  }), React.createElement(ParagraphDivider, null), React.createElement(Paragraph, {
    lines: 13,
    trail: "30%"
  }), React.createElement(ParagraphDivider, null), React.createElement(Skeleton, {
    height: "4%",
    width: "20%"
  }), React.createElement(ParagraphDivider, null), React.createElement(Paragraph, {
    lines: 8,
    trail: "50%"
  }));
};

var useStyles$4 = /*#__PURE__*/styles.makeStyles(function (theme) {
  return {
    root: {
      position: "relative",
      display: "flex",
      alignItems: "center",
      background: theme.palette.background.paper,
      padding: theme.spacing(0, 0.5, 0, 1),
      width: "180px"
    },
    dot: {
      flex: "0 0 auto",
      width: "8px",
      height: "8px",
      borderRadius: "100%"
    },
    text: {
      cursor: "pointer",
      flex: "1 1 auto",
      fontSize: "12px",
      padding: theme.spacing(0.125, 0.5, 0, 1),
      whiteSpace: "nowrap",
      overflow: "hidden",
      textOverflow: "ellipsis"
    },
    action: {
      flex: "0 0 auto",
      padding: 0
    }
  };
});

var AnnotationLabel = function AnnotationLabel(props) {
  var onClick = props.onClick,
      onDelete = props.onDelete,
      topic = props.topic,
      rest = _objectWithoutPropertiesLoose(props, ["onClick", "onDelete", "topic"]);

  var classes = useStyles$4();
  return React.createElement(core.Paper, Object.assign({
    square: true,
    className: classes.root
  }, rest), React.createElement("span", {
    className: classes.dot,
    style: {
      background: topicToColor(topic)
    }
  }), React.createElement("span", {
    className: classes.text,
    onClick: onClick
  }, topic), onDelete && React.createElement(core.IconButton, {
    className: classes.action,
    onClick: onDelete
  }, React.createElement(icons.Close, {
    fontSize: "small"
  })));
};

var MAX_ANNOTATION_LABELS_TO_SHOW = 5;
var MORE_BUTTON_HEIGHT = 20;
var ANNOTATION_LABEL_LEFT_OFFSET = -18;
var useStyles$5 = /*#__PURE__*/styles.makeStyles({
  root: {
    position: "absolute"
  },
  more: {
    width: "100%",
    height: MORE_BUTTON_HEIGHT + "px",
    borderRadius: 0,
    marginTop: "-3px"
  }
});
var AnnotationsGroup = function AnnotationsGroup(props) {
  var annotationsGroup = props.annotationsGroup,
      focusedAnnotationIndex = props.focusedAnnotationIndex,
      focusingAnnotation = props.focusingAnnotation,
      onAnnotationDelete = props.onAnnotationDelete,
      onAnnotationFocus = props.onAnnotationFocus;
  var classes = useStyles$5();
  var groupRef = React.useRef(null);

  var _React$useState = React.useState(false),
      expanded = _React$useState[0],
      setExpanded = _React$useState[1];

  var annotationsToShow = React.useMemo(function () {
    var annotations = annotationsGroup.annotations;
    if (expanded) return annotations; // Focused annotation is hidden

    if (focusingAnnotation && annotations.slice(MAX_ANNOTATION_LABELS_TO_SHOW).find(function (annotation) {
      return annotation._index === focusedAnnotationIndex;
    })) return annotations;
    return annotations.slice(0, MAX_ANNOTATION_LABELS_TO_SHOW);
  }, [annotationsGroup.annotations, expanded, focusedAnnotationIndex, focusingAnnotation]);
  var handleClickOutside = React.useCallback(function (event) {
    if (!groupRef || !groupRef.current) return;
    var targetElement = event.target;
    if (groupRef.current.contains(targetElement)) return;
    setExpanded(false);
    document.getElementsByClassName("Pages")[0].removeEventListener("click", handleClickOutside);
  }, [setExpanded]);
  var handleMoreButtonClick = React.useCallback(function () {
    setExpanded(true);
    document.getElementsByClassName("Pages")[0].addEventListener("click", handleClickOutside);
  }, [handleClickOutside, setExpanded]);
  return React.createElement(core.Box // Gotta love that non-sense
  // eslint-disable-next-line @typescript-eslint/ban-ts-comment
  // @ts-ignore see https://github.com/mui-org/material-ui/issues/17010
  , {
    // Gotta love that non-sense
    // eslint-disable-next-line @typescript-eslint/ban-ts-comment
    // @ts-ignore see https://github.com/mui-org/material-ui/issues/17010
    ref: groupRef,
    className: classes.root,
    style: {
      top: annotationsGroup.top
    }
  }, annotationsToShow.map(function (annotation, annotationKey) {
    return React.createElement(AnnotationLabel, {
      key: annotationKey,
      style: {
        marginLeft: focusingAnnotation && annotation._index === focusedAnnotationIndex ? ANNOTATION_LABEL_LEFT_OFFSET + "px" : "0",
        transition: "0.2s ease-in-out"
      },
      onClick: function onClick(event) {
        event.preventDefault();
        event.stopPropagation();
        onAnnotationFocus(annotation._index);
      },
      onDelete: onAnnotationDelete ? function (event) {
        event.preventDefault();
        event.stopPropagation();
        onAnnotationDelete(annotation);
      } : undefined,
      topic: annotation.topic
    });
  }), annotationsToShow.length < annotationsGroup.annotations.length && React.createElement(core.Button, {
    variant: "contained",
    disableElevation: true,
    className: classes.more,
    onClick: handleMoreButtonClick
  }, "More"));
};

var ANNOTATION_LABELS_WIDTH = 200;
var ANNOTATION_LABELS_MARGIN_LEFT = 6;
var ANNOTATION_LABEL_HEIGHT = 20;
var useStyles$6 = /*#__PURE__*/styles.makeStyles({
  root: {
    position: "relative",
    flex: "0 0 " + ANNOTATION_LABELS_WIDTH + "px",
    marginLeft: ANNOTATION_LABELS_MARGIN_LEFT + "px"
  }
});
var AnnotationLabels = function AnnotationLabels(props) {
  var annotations = props.annotations,
      focusedAnnotationIndex = props.focusedAnnotationIndex,
      focusingAnnotation = props.focusingAnnotation,
      onAnnotationFocus = props.onAnnotationFocus,
      onAnnotationDelete = props.onAnnotationDelete,
      originalPageHeight = props.originalPageHeight,
      pageNumber = props.pageNumber,
      pageWrapperHeight = props.pageWrapperHeight;
  var classes = useStyles$6();
  var annotationGroups = React.useMemo(function () {
    if (!pageWrapperHeight) return [];
    var filteredAnnotations = annotations.filter(function (annotation) {
      return annotation.pageStart === pageNumber;
    });
    if (!filteredAnnotations.length) return [];
    var containerHeight = pageWrapperHeight;
    var first = filteredAnnotations[0],
        rest = filteredAnnotations.slice(1);
    var firstTop = first.top * containerHeight / originalPageHeight;
    return rest.reduce(function (annotationGroups, annotation) {
      var lastGroupIndex = annotationGroups.length - 1;
      var lastGroup = annotationGroups[lastGroupIndex];
      var top = annotation.top * containerHeight / originalPageHeight;

      if (top <= lastGroup.bottom) {
        var bottom = lastGroup.annotations.length <= MAX_ANNOTATION_LABELS_TO_SHOW ? lastGroup.bottom + ANNOTATION_LABEL_HEIGHT : lastGroup.annotations.length === MAX_ANNOTATION_LABELS_TO_SHOW + 1 ? lastGroup.bottom + MORE_BUTTON_HEIGHT : lastGroup.bottom;
        annotationGroups[lastGroupIndex] = {
          top: lastGroup.top,
          bottom: bottom,
          annotations: lastGroup.annotations.concat([annotation])
        };
      } else {
        annotationGroups.push({
          top: top,
          bottom: top + ANNOTATION_LABEL_HEIGHT,
          annotations: [annotation]
        });
      }

      return annotationGroups;
    }, [{
      top: firstTop,
      bottom: firstTop + ANNOTATION_LABEL_HEIGHT,
      annotations: [first]
    }]);
  }, [annotations, originalPageHeight, pageNumber, pageWrapperHeight]);
  return React.createElement(core.Box, {
    className: classes.root
  }, annotationGroups.map(function (group, groupKey) {
    return React.createElement(AnnotationsGroup, {
      key: groupKey,
      annotationsGroup: group,
      focusedAnnotationIndex: focusedAnnotationIndex,
      focusingAnnotation: focusingAnnotation,
      onAnnotationDelete: onAnnotationDelete,
      onAnnotationFocus: onAnnotationFocus
    });
  }));
};

var WHOLE_PAGE_ZOOM = "whole-page-zoom";
var useStyles$7 = /*#__PURE__*/styles.makeStyles(function (theme) {
  return {
    root: {
      display: "block",
      marginLeft: theme.spacing(1.5)
    },
    skeletonAnnotations: {
      position: "relative",
      flex: "0 0 " + ANNOTATION_LABELS_WIDTH + "px",
      marginLeft: ANNOTATION_LABELS_MARGIN_LEFT + "px"
    },
    skeletonAnnotationDot: {
      background: theme.palette.info.main,
      flex: "0 0 auto",
      width: "8px",
      height: "8px",
      borderRadius: "100%"
    },
    skeletonAnnotationLabel: {
      position: "relative",
      display: "flex",
      alignItems: "center",
      background: theme.palette.background.paper,
      padding: theme.spacing(0, 0.5, 0, 1),
      height: "20px",
      width: "180px"
    },
    skeletonAnnotationText: {
      cursor: "pointer",
      flex: "1 1 auto",
      fontSize: "12px",
      margin: theme.spacing(0.125, 0.5, 0, 1)
    },
    wrapper: {
      display: "flex",
      flexFlow: "row nowrap",
      margin: "0 auto"
    }
  };
});

var getPageDimensions = function getPageDimensions(containerWidth, containerHeight, originalPageWidth, originalPageHeight, zoom) {
  var pageWrapperHeight;
  var pageWrapperWidth;

  if (zoom == WHOLE_PAGE_ZOOM) {
    pageWrapperHeight = containerHeight;
    pageWrapperWidth = containerHeight * (originalPageWidth / originalPageHeight) + (ANNOTATION_LABELS_WIDTH + ANNOTATION_LABELS_MARGIN_LEFT);
  } else {
    var pageImageWrapperWidth = containerWidth * (parseInt(zoom) / 100) - (ANNOTATION_LABELS_WIDTH + ANNOTATION_LABELS_MARGIN_LEFT);
    pageWrapperHeight = pageImageWrapperWidth * (originalPageHeight / originalPageWidth);
    pageWrapperWidth = pageImageWrapperWidth + (ANNOTATION_LABELS_WIDTH + ANNOTATION_LABELS_MARGIN_LEFT);
  }

  return [pageWrapperWidth, pageWrapperHeight];
};

var EmptyPage = function EmptyPage(props) {
  var containerWidth = props.containerWidth,
      containerHeight = props.containerHeight,
      originalPageWidth = props.originalPageWidth,
      originalPageHeight = props.originalPageHeight,
      zoom = props.zoom;
  var classes = useStyles$7();

  var _React$useState = React.useState(0),
      pageWrapperHeight = _React$useState[0],
      setPageWrapperHeight = _React$useState[1];

  var _React$useState2 = React.useState(0),
      pageWrapperWidth = _React$useState2[0],
      setPageWrapperWidth = _React$useState2[1];

  var setPageDimensions = React.useCallback(function () {
    var _getPageDimensions = getPageDimensions(containerWidth, containerHeight, originalPageWidth, originalPageHeight, zoom),
        pageWrapperWidth = _getPageDimensions[0],
        pageWrapperHeight = _getPageDimensions[1];

    setPageWrapperWidth(pageWrapperWidth);
    setPageWrapperHeight(pageWrapperHeight);
  }, [containerWidth, containerHeight, originalPageWidth, originalPageHeight, zoom]);
  React.useEffect(function () {
    setPageDimensions();
  }, [setPageDimensions]);
  return React.createElement(core.ListItem, {
    className: classes.root,
    disableGutters: true,
    style: {
      width: "calc(100% - 15px)"
    }
  }, React.createElement(core.Box, {
    className: classes.wrapper,
    style: {
      width: pageWrapperWidth + "px",
      height: pageWrapperHeight + "px"
    }
  }, React.createElement(PageSkeleton, null), React.createElement(core.Box, {
    className: classes.skeletonAnnotations
  }, React.createElement(core.Box, {
    position: "absolute"
  }, [].concat(Array(5)).map(function (_, idx) {
    return React.createElement(core.Paper, {
      square: true,
      key: idx,
      className: classes.skeletonAnnotationLabel
    }, React.createElement("span", {
      className: classes.skeletonAnnotationDot
    }), React.createElement(Skeleton, {
      className: classes.skeletonAnnotationText,
      height: "60%",
      width: "100%"
    }));
  })))));
};

var Page = function Page(props) {
  var annotations = props.annotations,
      containerHeight = props.containerHeight,
      containerWidth = props.containerWidth,
      focusedAnnotationIndex = props.focusedAnnotationIndex,
      focusingAnnotation = props.focusingAnnotation,
      imageURL = props.imageURL,
      load = props.load,
      onAnnotationDelete = props.onAnnotationDelete,
      onFocusedAnnotationIndexChange = props.onFocusedAnnotationIndexChange,
      onFocusingAnnotationChange = props.onFocusingAnnotationChange,
      onSelectionStart = props.onSelectionStart,
      onSelectionEnd = props.onSelectionEnd,
      originalPageWidth = props.originalPageWidth,
      originalPageHeight = props.originalPageHeight,
      pageNumber = props.pageNumber,
      mouseSelection = props.mouseSelection,
      searchResults = props.searchResults,
      selection = props.selection,
      tokensURL = props.tokensURL,
      zoom = props.zoom;
  var classes = useStyles$7();

  var _React$useState3 = React.useState(0),
      pageWrapperHeight = _React$useState3[0],
      setPageWrapperHeight = _React$useState3[1];

  var _React$useState4 = React.useState(0),
      pageWrapperWidth = _React$useState4[0],
      setPageWrapperWidth = _React$useState4[1];

  var pageRef = React.useRef(null);
  var setPageDimensions = React.useCallback(function () {
    var _getPageDimensions2 = getPageDimensions(containerWidth, containerHeight, originalPageWidth, originalPageHeight, zoom),
        pageWrapperWidth = _getPageDimensions2[0],
        pageWrapperHeight = _getPageDimensions2[1];

    setPageWrapperWidth(pageWrapperWidth);
    setPageWrapperHeight(pageWrapperHeight);
  }, [containerWidth, containerHeight, originalPageWidth, originalPageHeight, zoom]);
  React.useEffect(function () {
    if (!pageRef || !pageRef.current) return;
    setPageDimensions();
  }, [setPageDimensions]);
  var handleFocusAnnotation = React.useCallback(function (annotationIndex) {
    onFocusedAnnotationIndexChange(annotationIndex);
    onFocusingAnnotationChange(true);
  }, [onFocusingAnnotationChange, onFocusedAnnotationIndexChange]);
  return React.createElement(core.ListItem, {
    className: classes.root,
    ref: pageRef,
    disableGutters: true,
    style: {
      width: "calc(100% - 15px)"
    }
  }, React.createElement(core.Box, {
    className: classes.wrapper,
    style: {
      width: pageWrapperWidth + "px",
      height: pageWrapperHeight + "px"
    }
  }, load ? React.createElement(Viewbox, {
    annotations: annotations,
    searchResults: searchResults,
    focusedAnnotationIndex: focusedAnnotationIndex,
    focusingAnnotation: focusingAnnotation,
    imageURL: imageURL,
    onAnnotationFocus: handleFocusAnnotation,
    onSelectionStart: onSelectionStart,
    onSelectionEnd: onSelectionEnd,
    originalPageHeight: originalPageHeight,
    originalPageWidth: originalPageWidth,
    pageNumber: pageNumber,
    mouseSelection: mouseSelection,
    selection: selection,
    tokensURL: tokensURL
  }) : React.createElement(EmptyViewbox, null), React.createElement(AnnotationLabels, {
    annotations: annotations,
    focusedAnnotationIndex: focusedAnnotationIndex,
    focusingAnnotation: focusingAnnotation,
    originalPageHeight: originalPageHeight,
    onAnnotationDelete: onAnnotationDelete,
    onAnnotationFocus: handleFocusAnnotation,
    pageNumber: pageNumber,
    pageWrapperHeight: pageWrapperHeight
  })));
};

var smoothScroll = function smoothScroll(container, destination, duration, done) {
  var _destination$endX, _destination$endY;

  if (duration === void 0) {
    duration = 200;
  }

  var startX = container.scrollLeft;
  var startY = container.scrollTop;
  var finalX = (_destination$endX = destination.endX) != null ? _destination$endX : startX;
  var finalY = (_destination$endY = destination.endY) != null ? _destination$endY : startY;
  var distanceX = finalX - startX;
  var distanceY = finalY - startY;
  var startTime = performance.now();

  var easeOut = function easeOut(t) {
    return (2 - t) * t;
  };

  var scroll = function scroll(currentTime) {
    var timeProgress = (currentTime - startTime) / duration;
    var relativeProgress = easeOut(timeProgress);
    var distanceProgressX = relativeProgress * distanceX;
    var distanceProgressY = relativeProgress * distanceY;

    if (timeProgress < 1.0) {
      container.scrollLeft = startX + distanceProgressX;
      container.scrollTop = startY + distanceProgressY;
      window.requestAnimationFrame(scroll);
    } else {
      container.scrollLeft = finalX;
      container.scrollTop = finalY;
      if (done) done();
    }
  };

  window.requestAnimationFrame(scroll);
};

var useStyles$8 = /*#__PURE__*/styles.makeStyles({
  root: function root(_ref) {
    var width = _ref.width;
    return {
      overflow: "auto",
      maxWidth: width,
      width: "100%",
      height: "100%"
    };
  }
});

var annotationsPerPage = function annotationsPerPage(annotations) {
  var pages = new Map();
  annotations.forEach(function (annotation) {
    for (var i = annotation.pageStart; i <= annotation.pageEnd; i++) {
      var pageAnnotation = _extends({}, annotation);

      if (i === annotation.pageStart && i !== annotation.pageEnd) {
        pageAnnotation.characterEnd = Number.POSITIVE_INFINITY;
      } else if (i === annotation.pageEnd && i !== annotation.pageStart) {
        pageAnnotation.characterStart = Number.NEGATIVE_INFINITY;
      } else if (i > annotation.pageStart && i < annotation.pageEnd) {
        pageAnnotation.characterStart = Number.NEGATIVE_INFINITY;
        pageAnnotation.characterEnd = Number.POSITIVE_INFINITY;
      }

      if (!pages.get(i)) {
        pages.set(i, []);
      }

      pages.get(i).push(pageAnnotation);
    }
  });
  return pages;
};

var searchResultsPerPage = function searchResultsPerPage(searchResults) {
  var pages = new Map();
  searchResults.forEach(function (searchResult) {
    for (var i = searchResult.pageStart; i <= searchResult.pageEnd; i++) {
      var pageSearchResult = _extends({}, searchResult);

      if (i === searchResult.pageStart && i !== searchResult.pageEnd) {
        pageSearchResult.characterEnd = Number.POSITIVE_INFINITY;
      } else if (i === searchResult.pageEnd && i !== searchResult.pageStart) {
        pageSearchResult.characterStart = Number.NEGATIVE_INFINITY;
      } else if (i > searchResult.pageStart && i < searchResult.pageEnd) {
        pageSearchResult.characterStart = Number.NEGATIVE_INFINITY;
        pageSearchResult.characterEnd = Number.POSITIVE_INFINITY;
      }

      if (!pages.get(i)) {
        pages.set(i, []);
      }

      pages.get(i).push(pageSearchResult);
    }
  });
  return pages;
};

var selectionPerPage = function selectionPerPage(selectionStart, selectionEnd) {
  var pages = new Map();
  if (!selectionStart || !selectionEnd) return pages;
  var selection = {
    indexStart: selectionStart.index,
    indexEnd: selectionEnd.index,
    pageStart: selectionStart.page,
    pageEnd: selectionEnd.page
  };

  for (var i = selection.pageStart; i <= selection.pageEnd; i++) {
    var pageSelection = {
      indexStart: selection.indexStart,
      indexEnd: selection.indexEnd
    };

    if (i === selection.pageStart && i !== selection.pageEnd) {
      pageSelection.indexEnd = Number.POSITIVE_INFINITY;
    } else if (i === selection.pageEnd && i !== selection.pageStart) {
      pageSelection.indexStart = Number.NEGATIVE_INFINITY;
    } else if (i > selection.pageStart && i < selection.pageEnd) {
      pageSelection.indexStart = Number.NEGATIVE_INFINITY;
      pageSelection.indexEnd = Number.POSITIVE_INFINITY;
    }

    pages.set(i, pageSelection);
  }

  return pages;
};

var pageInView = function pageInView(container) {
  var pageNumber = 0;
  var children = container.children;
  var containerBCR = container.getBoundingClientRect();
  var containerMiddle = containerBCR.top + containerBCR.height / 2;
  var pageBCR;

  do {
    pageBCR = children[pageNumber].getBoundingClientRect();
    pageNumber++;
  } while (pageBCR.bottom < containerMiddle);

  return pageNumber;
};

var Pages = function Pages(props, ref) {
  var annotations = props.annotations,
      searchResults = props.searchResults,
      currentPage = props.currentPage,
      topics = props.topics,
      focusedAnnotationIndex = props.focusedAnnotationIndex,
      focusingAnnotation = props.focusingAnnotation,
      lazyLoadingWindow = props.lazyLoadingWindow,
      loading = props.loading,
      onAnnotationCreate = props.onAnnotationCreate,
      onAnnotationDelete = props.onAnnotationDelete,
      onFocusedAnnotationIndexChange = props.onFocusedAnnotationIndexChange,
      onFocusingAnnotationChange = props.onFocusingAnnotationChange,
      onPageChange = props.onPageChange,
      pages = props.pages,
      pagesWidth = props.pagesWidth,
      pagesHeight = props.pagesHeight,
      zoom = props.zoom;
  var classes = useStyles$8({
    width: pagesWidth
  });

  var _React$useState = React.useState([0, 0]),
      mouseStart = _React$useState[0],
      setMouseStart = _React$useState[1];

  var _React$useState2 = React.useState(null),
      mouseEnd = _React$useState2[0],
      setMouseEnd = _React$useState2[1];

  var _React$useState3 = React.useState(false),
      selecting = _React$useState3[0],
      setSelecting = _React$useState3[1];

  var _React$useState4 = React.useState(null),
      selectionStart = _React$useState4[0],
      setSelectionStart = _React$useState4[1];

  var _React$useState5 = React.useState(null),
      selectionEnd = _React$useState5[0],
      setSelectionEnd = _React$useState5[1];

  var _React$useState6 = React.useState(false),
      scrolling = _React$useState6[0],
      setScrolling = _React$useState6[1];

  var _React$useState7 = React.useState([0, 0]),
      scrollPosition = _React$useState7[0],
      setScrollPosition = _React$useState7[1];

  var pagesEl = React.useRef(null);
  var pageAnnotations = React.useMemo(function () {
    return annotationsPerPage(annotations);
  }, [annotations]);
  var pageSearchResults = React.useMemo(function () {
    return searchResultsPerPage(searchResults);
  }, [searchResults]);
  var pageSelection = React.useMemo(function () {
    return selectionPerPage(selectionStart, selectionEnd);
  }, [selectionStart, selectionEnd]);
  var mouseSelection = React.useMemo(function () {
    if (mouseStart && mouseEnd) {
      return mouseStart[1] < mouseEnd[1] ? [mouseStart, mouseEnd] : [mouseEnd, mouseStart];
    } else {
      return null;
    }
  }, [mouseStart, mouseEnd]);

  var mousePosition = function mousePosition(event) {
    if (pagesEl && pagesEl.current) {
      var pagesBCR = pagesEl.current.getBoundingClientRect();
      return [event.clientX - pagesBCR.left + pagesEl.current.scrollLeft, event.clientY - pagesBCR.top + pagesEl.current.scrollTop];
    }

    return [0, 0];
  }; // When new document, reinitialzes selection variables


  React.useEffect(function () {
    setMouseStart([0, 0]);
    setMouseEnd(null);
    setSelecting(false);
    setSelectionStart(null);
    setSelectionEnd(null);
  }, [pages]);
  var handleMouseMove = React.useCallback(function (event) {
    var newMouseEnd = mousePosition(event);

    if (!lodash.isEqual(mouseEnd, newMouseEnd)) {
      setMouseEnd(newMouseEnd);
    }
  }, [mouseEnd, setMouseEnd]);
  var handleMouseUp = React.useCallback(function () {
    setSelecting(false);
    window.removeEventListener("mouseup", handleMouseUp);
    window.removeEventListener("mousemove", handleMouseMove);
  }, [handleMouseMove]);
  var handleMouseDown = React.useCallback(function (event) {
    if (event.button !== 0) {
      // clicked something else than the main button
      return;
    } // Return if the user selected a topic


    var targetElement = event.target;

    if (targetElement.matches(".Picker, .Picker *")) {
      return;
    }

    var position = mousePosition(event); // If a user holds shift + click somewhere,
    // adjust the mouseEnd

    if (event.shiftKey && mouseStart) {
      setMouseEnd(position);
      return;
    } // Reset selection


    setSelectionStart(null);
    setSelectionEnd(null);
    setMouseStart(position);
    setMouseEnd(null);
    setSelecting(true); // Only listen to mouseup and mousemove when the mouse button is pressed

    window.addEventListener("mouseup", handleMouseUp);
    window.addEventListener("mousemove", handleMouseMove);
  }, [handleMouseMove, handleMouseUp, mouseStart]);
  var handleClick = React.useCallback(function () {
    onFocusingAnnotationChange(false);
  }, [onFocusingAnnotationChange]);
  var handleSelectionStart = React.useCallback(function (selection) {
    if (!lodash.isEqual(selection, selectionStart)) {
      setSelectionStart(selection);
    }
  }, [setSelectionStart, selectionStart]);
  var handleSelectionEnd = React.useCallback(function (selection) {
    if (!lodash.isEqual(selection, selectionEnd)) {
      setSelectionEnd(selection);
    }
  }, [setSelectionEnd, selectionEnd]);
  var handleScroll = React.useCallback(function () {
    if (!pagesEl || !pagesEl.current) return;
    onPageChange(pageInView(pagesEl.current));
    var newScrollLeft = pagesEl.current.scrollLeft;
    var newScrollTop = pagesEl.current.scrollTop; // If user is selecting and scrolling, it won't send a new MouseMove event
    // so we need to adjust the MousePosition based on the delta of the scroll

    if (selecting && mouseStart) {
      var _ref2 = mouseEnd || mouseStart,
          mouseX = _ref2[0],
          mouseY = _ref2[1];

      var scrollLeft = scrollPosition[0],
          scrollTop = scrollPosition[1];
      setMouseEnd([mouseX + (newScrollLeft - scrollLeft), mouseY + (newScrollTop - scrollTop)]);
    }

    setScrollPosition([newScrollLeft, newScrollTop]);
  }, [mouseEnd, mouseStart, onPageChange, scrollPosition, selecting, setMouseEnd, setScrollPosition]);
  var handleAnnotation = React.useCallback(function (topic) {
    if (!selectionStart || !selectionEnd || !onAnnotationCreate) return;
    onAnnotationCreate({
      characterStart: selectionStart.token.characterStart,
      characterEnd: selectionEnd.token.characterEnd,
      pageStart: selectionStart.page,
      pageEnd: selectionEnd.page,
      top: selectionStart.token.boundingBox.top,
      left: selectionStart.token.boundingBox.left,
      topic: topic
    }); // Clear mouse and selection

    setMouseStart([0, 0]);
    setMouseEnd([0, 0]);
    setSelectionStart(null);
    setSelectionEnd(null);
  }, [onAnnotationCreate, selectionStart, selectionEnd]);
  React.useImperativeHandle(ref, function () {
    return {
      scrollToPage: function scrollToPage(pageNumber) {
        if (!pagesEl.current || currentPage === pageNumber) return;
        var pageEl = pagesEl.current.children[pageNumber - 1];
        var pagesBCR = pagesEl.current.getBoundingClientRect();
        var pageBCR = pageEl.getBoundingClientRect();
        setScrolling(true);
        smoothScroll(pagesEl.current, {
          endY: pageBCR.top - pagesBCR.top + pagesEl.current.scrollTop
        }, 200, function () {
          return setScrolling(false);
        });
      },
      scrollToLocation: function scrollToLocation(pageNumber, originalLeft, originalTop) {
        if (!pagesEl.current) return;
        var page = pages[pageNumber - 1];
        var pageEl = pagesEl.current.children[pageNumber - 1];
        var pagesBCR = pagesEl.current.getBoundingClientRect();
        var pageBCR = pageEl.getBoundingClientRect();
        var pageX = pageBCR.left - pagesBCR.left + pagesEl.current.scrollLeft;
        var pageY = pageBCR.top - pagesBCR.top + pagesEl.current.scrollTop;
        var locationX = originalLeft * (pageBCR.width / page.originalWidth);
        var locationY = originalTop * (pageBCR.height / page.originalHeight);
        setScrolling(true);
        console.log("Annotation2", {
          endX: pageX + locationX - 20,
          endY: pageY + locationY - 20
        });
        smoothScroll(pagesEl.current, {
          endX: pageX + locationX - 20,
          endY: pageY + locationY - 20
        }, 200, function () {
          return setScrolling(false);
        });
      },
      scrollToAnnotation: function scrollToAnnotation(annotationIndex) {
        console.log("Annotation", annotationIndex);
        if (!pagesEl.current) return;
        console.log("Annotation2");
        var annotation = annotations[annotationIndex];
        console.log("Annotation2", annotation);
        this.scrollToLocation(annotation.pageStart, annotation.left, annotation.top);
      }
    };
  }, [annotations, currentPage, pages]);
  return React.createElement(core.List, {
    ref: pagesEl,
    className: clsx("Pages", classes.root),
    onMouseDown: handleMouseDown,
    onClick: handleClick,
    onScroll: handleScroll
  }, loading ? React.createElement(EmptyPage, {
    containerHeight: pagesHeight,
    containerWidth: pagesWidth,
    originalPageHeight: 11,
    originalPageWidth: 8.5,
    zoom: zoom
  }) : pages.map(function (page, index) {
    var _pageAnnotations$get, _pageSearchResults$ge;

    return React.createElement(Page, {
      key: index,
      annotations: (_pageAnnotations$get = pageAnnotations.get(index + 1)) != null ? _pageAnnotations$get : [],
      containerHeight: pagesHeight,
      containerWidth: pagesWidth,
      focusedAnnotationIndex: focusedAnnotationIndex,
      focusingAnnotation: focusingAnnotation,
      imageURL: page.imageURL,
      load: !scrolling && index + 1 >= currentPage - lazyLoadingWindow && index + 1 <= currentPage + lazyLoadingWindow,
      onAnnotationDelete: onAnnotationDelete,
      onFocusedAnnotationIndexChange: onFocusedAnnotationIndexChange,
      onFocusingAnnotationChange: onFocusingAnnotationChange,
      onSelectionStart: handleSelectionStart,
      onSelectionEnd: handleSelectionEnd,
      originalPageHeight: page.originalHeight,
      originalPageWidth: page.originalWidth,
      pageNumber: index + 1,
      mouseSelection: mouseSelection,
      searchResults: (_pageSearchResults$ge = pageSearchResults.get(index + 1)) != null ? _pageSearchResults$ge : [],
      selection: pageSelection.get(index + 1) || null,
      tokensURL: page.tokensURL,
      zoom: zoom
    });
  }), onAnnotationCreate && selectionStart && selectionEnd && React.createElement(Picker, {
    topics: topics,
    onAnnotation: handleAnnotation
  }));
};

var Pages$1 = /*#__PURE__*/React.forwardRef(Pages);

var DEFAULT_ZOOM = "75%";
var DEFAULT_LAZY_LOADING_WINDOW = 2;
var generateClassName = /*#__PURE__*/core.createGenerateClassName({
  disableGlobal: true,
  seed: "dv"
});
var useStyles$9 = /*#__PURE__*/core.makeStyles({
  root: {
    display: "flex",
    flexFlow: "column nowrap",
    overflow: "hidden",
    height: "100%",
    width: "100%",
    background: "#F9FBFF",
    padding: 0,
    margin: 0
  },
  appBar: {
    zIndex: 1201
  },
  container: {
    display: "flex",
    flexFlow: "row nowrap",
    overflow: "hidden",
    height: "100%",
    width: "100%",
    padding: 0,
    margin: 0
  },
  viewer: {
    flex: "1 1 auto",
    padding: 0,
    margin: 0
  }
});

var DocumentViewer = function DocumentViewer(props, ref) {
  var id = props.id,
      annotations = props.annotations,
      _props$lazyLoadingWin = props.lazyLoadingWindow,
      lazyLoadingWindow = _props$lazyLoadingWin === void 0 ? DEFAULT_LAZY_LOADING_WINDOW : _props$lazyLoadingWin,
      loading = props.loading,
      name = props.name,
      onAnnotationCreate = props.onAnnotationCreate,
      onAnnotationDelete = props.onAnnotationDelete,
      onClose = props.onClose,
      onNextDocument = props.onNextDocument,
      onPreviousDocument = props.onPreviousDocument,
      pages = props.pages,
      _props$topics = props.topics,
      topics = _props$topics === void 0 ? [] : _props$topics,
      _props$searchResults = props.searchResults,
      searchResults = _props$searchResults === void 0 ? [] : _props$searchResults,
      Summary = props.Summary,
      summaryProps = props.summaryProps,
      _props$muiClassGenera = props.muiClassGenerator,
      muiClassGenerator = _props$muiClassGenera === void 0 ? generateClassName : _props$muiClassGenera;
  var classes = useStyles$9(props);

  var _React$useState = React.useState(1),
      currentPage = _React$useState[0],
      setCurrentPage = _React$useState[1];

  var _React$useState2 = React.useState(false),
      focusingAnnotation = _React$useState2[0],
      setFocusingAnnotation = _React$useState2[1];

  var _React$useState3 = React.useState(1),
      focusedAnnotationIndex = _React$useState3[0],
      setFocusedAnnotationIndex = _React$useState3[1];

  var containerEl = React.useRef(null);
  var pagesEl = React.useRef(null);
  var summaryEl = React.useRef(null);

  var _React$useState4 = React.useState(DEFAULT_ZOOM),
      zoom = _React$useState4[0],
      setZoom = _React$useState4[1];

  var _React$useState5 = React.useState(0),
      pagesHeight = _React$useState5[0],
      setPagesHeight = _React$useState5[1];

  var _React$useState6 = React.useState(0),
      pagesWidth = _React$useState6[0],
      setPagesWidth = _React$useState6[1];

  var onSummaryResize = React.useCallback(function (width, height) {
    if (pagesWidth != width && containerEl.current) {
      setPagesWidth(containerEl.current.offsetWidth - width);
    }

    if (pagesHeight != height && containerEl.current) {
      setPagesHeight(containerEl.current.offsetHeight);
    }
  }, [pagesWidth, pagesHeight]);
  var onWindowResize = React.useCallback(function () {
    if (!containerEl.current) return;
    var width = containerEl.current.offsetWidth;
    var height = containerEl.current.offsetHeight;

    if (summaryEl.current) {
      var summaryWidth = summaryEl.current.offsetWidth;
      width -= summaryWidth;
    }

    setPagesWidth(width);
    setPagesHeight(height);
  }, []);
  var resizeObserver = new ResizeObserver(function (entries) {
    window.requestAnimationFrame(function () {
      entries.forEach(function (entry) {
        onSummaryResize(entry.contentRect.width, entry.contentRect.height);
      });
    });
  }); // Track resize of Window

  React.useEffect(function () {
    onWindowResize();
    window.addEventListener("resize", onWindowResize);
    return function () {
      window.removeEventListener("resize", onWindowResize);
    };
  }, []); // Track resize of Summary

  React.useEffect(function () {
    if (summaryEl.current != null) {
      resizeObserver.observe(summaryEl.current);
    }
  }, [summaryEl.current]);
  var handleZoomChange = React.useCallback(function (newZoom) {
    setZoom(newZoom);
  }, []);

  var sortAnnotations = function sortAnnotations(annotations) {
    return annotations.sort(function (a, b) {
      if (a.characterStart > b.characterStart) return 1;
      if (a.characterStart < b.characterStart) return -1;
      if (a.characterEnd > b.characterEnd) return 1;
      if (a.characterEnd < b.characterEnd) return -1;
      return 0;
    });
  };

  var indexAnnotations = function indexAnnotations(annotations) {
    return sortAnnotations(annotations).map(function (annotation, index) {
      return _extends({}, annotation, {
        _index: index + 1
      });
    });
  }; // When deleting annotations, it shouldn't be out of bounds


  React.useEffect(function () {
    if (focusedAnnotationIndex > annotations.length) {
      setFocusedAnnotationIndex(annotations.length - 1);
    }
  }, [annotations, focusedAnnotationIndex]); // When a different document, it should scroll back to the first page

  React.useEffect(function () {
    var _pagesEl$current;

    pagesEl == null ? void 0 : (_pagesEl$current = pagesEl.current) == null ? void 0 : _pagesEl$current.scrollToPage(1);
  }, [id]);
  var handleNavigationIndexChange = React.useCallback(function (navigationIndex) {
    var _pagesEl$current2;

    if (navigationIndex < 1 || navigationIndex > pages.length) return;
    pagesEl == null ? void 0 : (_pagesEl$current2 = pagesEl.current) == null ? void 0 : _pagesEl$current2.scrollToPage(navigationIndex);
  }, [annotations, pages]);
  var handlePageChange = React.useCallback(function (pageNumber) {
    if (pageNumber === currentPage || pageNumber < 1 || pageNumber > pages.length) return;
    setCurrentPage(pageNumber);
  }, [currentPage, pages]);
  var handleFocusedAnnotationChange = React.useCallback(function (annotationIndex) {
    if (focusedAnnotationIndex === annotationIndex || annotationIndex < 1 || annotationIndex > annotations.length) return;
    setFocusedAnnotationIndex(annotationIndex);
  }, [annotations, focusedAnnotationIndex]);
  var handleFocusingAnnotationChange = React.useCallback(function (focusing) {
    if (focusingAnnotation === focusing) return;
    setFocusingAnnotation(focusing);
  }, [focusingAnnotation]);
  var handleAnnotationDelete = onAnnotationDelete ? function (annotation) {
    var clonedAnnotation = _objectWithoutPropertiesLoose(annotation, ["_index"]);

    onAnnotationDelete(clonedAnnotation);
  } : undefined;
  return React.createElement(core.StylesProvider, {
    generateClassName: muiClassGenerator
  }, React.createElement(core.Box, {
    ref: ref,
    className: classes.root,
    tabIndex: -1
  }, React.createElement(core.AppBar, {
    className: classes.appBar,
    position: "relative",
    elevation: 1
  }, React.createElement(NavBar, {
    documentName: name,
    navigationIndex: currentPage,
    navigationTotal: pages.length,
    onNavigationIndexChange: handleNavigationIndexChange,
    onClose: onClose,
    onNextDocument: onNextDocument,
    onPreviousDocument: onPreviousDocument
  })), React.createElement(core.Box, {
    ref: containerEl,
    className: classes.container
  }, Summary && React.createElement(Summary, Object.assign({
    ref: summaryEl
  }, summaryProps, {
    zoom: zoom,
    onZoomChange: handleZoomChange,
    viewerRef: pagesEl
  })), React.createElement(core.Box, {
    className: classes.viewer
  }, React.createElement(Pages$1, {
    ref: pagesEl,
    annotations: indexAnnotations(annotations),
    currentPage: currentPage,
    focusedAnnotationIndex: focusedAnnotationIndex,
    focusingAnnotation: focusingAnnotation,
    lazyLoadingWindow: lazyLoadingWindow,
    loading: loading,
    onAnnotationCreate: onAnnotationCreate,
    onAnnotationDelete: handleAnnotationDelete,
    onFocusedAnnotationIndexChange: handleFocusedAnnotationChange,
    onFocusingAnnotationChange: handleFocusingAnnotationChange,
    onPageChange: handlePageChange,
    pages: pages,
    pagesHeight: pagesHeight,
    pagesWidth: pagesWidth,
    topics: topics,
    searchResults: searchResults,
    zoom: zoom
  })))));
};

var DocumentViewer$1 = /*#__PURE__*/React.memo( /*#__PURE__*/React.forwardRef(DocumentViewer), lodash.isEqual);

exports.default = DocumentViewer$1;
//# sourceMappingURL=document-viewer.cjs.development.js.map
